package com.abi.qms.platform.service.impl;


import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.map.MapUtil;
import com.abi.qms.platform.dao.entity.InspectionCase;
import com.abi.qms.platform.dao.entity.InspectionCaseBoxCode;
import com.abi.qms.platform.dao.enums.CaseNumberGenerateStrategyEnum;
import com.abi.qms.platform.dao.enums.ReqChannelTypeEnum;
import com.abi.qms.platform.dao.mapper.InspectionCaseBoxCodeMapper;
import com.abi.qms.platform.dao.mapper.InspectionCaseMapper;
import com.abi.qms.platform.dao.vo.result.InspectionCaseDetailVO;
import com.abi.qms.platform.dao.vo.result.ListInspectionCaseVO;
import com.abi.qms.platform.dao.vo.result.ListRoleVO;
import com.abi.qms.platform.dto.req.AddInspectionCaseReq;
import com.abi.qms.platform.dto.req.EditInspectionCaseReq;
import com.abi.qms.platform.dto.req.PageListInspectionCaseReq;
import com.abi.qms.platform.dto.res.GetInspectionCaseInfoRes;
import com.abi.qms.platform.dto.res.ListRoleRes;
import com.abi.qms.platform.dto.res.PageListInspectionCaseRes;
import com.abi.qms.platform.infrastructure.util.AssertUtil;
import com.abi.qms.platform.infrastructure.util.PageUtil;
import com.abi.qms.platform.service.IInspectionCaseService;
import com.abi.qms.platform.service.InspectionCaseNumberService;
import com.abi.task.common.api.exception.BusinessException;
import com.abi.task.common.utils.PojoConverterUtils;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.validation.constraints.NotNull;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.Map;
import java.util.function.Consumer;
import java.util.function.Predicate;
import java.util.stream.Collectors;


/**
 * 稽查案件 Service业务层处理
 *
 * @author ZhangZhao
 * @date: 2021-07-19
 */
@Service
public class InspectionCaseServiceImpl implements IInspectionCaseService {


    @Autowired
    private InspectionCaseMapper inspectionCaseMapper;

    @Autowired
    private InspectionCaseBoxCodeMapper inspectionCaseBoxCodeMapper;

    @Autowired
    private InspectionCaseNumberService caseNumberService;

    /**
     * 新增案件
     *
     * @param addReq 新增请求
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public void addInspectionCase(AddInspectionCaseReq addReq) {

        //============先校验箱码============
        List<AddInspectionCaseReq.BoxCode> involvedBoxCodes = addReq.getInvolvedBoxCodes();
        List<String> boxCodeStrList = involvedBoxCodes.stream().map(AddInspectionCaseReq.BoxCode::getBoxCode).collect(Collectors.toList());
        //校验箱码信息  校验通过返回箱码对应流向信息
        Map flowInfo = this.validatedBoxCode(boxCodeStrList);

        //============案件编号============
        String caseNumber = addReq.getCaseNumber();
        //生成
        Integer generateStrategy = addReq.getCaseNumberGenerateStrategy();
        if (CaseNumberGenerateStrategyEnum.AUTO.is(generateStrategy)) {
            caseNumber = this.autoGenerateCaseNumber(addReq.getProvinceId(), addReq.getCityId());
        }

        //查看编号是否存在
        QueryWrapper<InspectionCase> inspectionCaseQueryWrapper = new QueryWrapper<>();
        inspectionCaseQueryWrapper.eq("case_number", caseNumber);
        inspectionCaseQueryWrapper.eq("is_delete", 0);
        List<InspectionCase> inspectionCaseList = inspectionCaseMapper.selectList(inspectionCaseQueryWrapper);
        AssertUtil.isEmpty(inspectionCaseList, "当前编号已存在！");

        //============保存案件============

        InspectionCase inspectionCase = new InspectionCase();
        //TODO 设置流入 流出经销商 从 flowInfo 中取出
        inspectionCase.setOutAgencyCode("T0001");
        inspectionCase.setOutAgencyName("T_京城XXX超市");
        inspectionCase.setInAgencyCode("T0002");
        inspectionCase.setInAgencyName("T_京城XXX便利店");

        BeanUtil.copyProperties(addReq, inspectionCase);
        inspectionCase.setBoxCodeCount((long) addReq.getInvolvedBoxCodes().size());
        inspectionCase.setCaseNumber(caseNumber);
        //设置渠道=>后台
        inspectionCase.setChannel(ReqChannelTypeEnum.BACKSTAGE.getCode());
        inspectionCaseMapper.insert(inspectionCase);

        //============保存涉案箱码============
        //转换类型
        List<InspectionCaseBoxCode> inspectionCaseBoxCodeList = involvedBoxCodes.stream()
                .parallel().map(s -> {
                    InspectionCaseBoxCode inspectionCaseBoxCode = new InspectionCaseBoxCode();
                    BeanUtil.copyProperties(s, inspectionCaseBoxCode);
                    inspectionCaseBoxCode.setInspectionCaseId(inspectionCase.getId());
                    return inspectionCaseBoxCode;
                }).collect(Collectors.toList());

        //============保存【案件-流向】关联============


        //批量保存
        inspectionCaseBoxCodeMapper.insertBatch(inspectionCaseBoxCodeList);

    }


    /**
     * 分页列表查询
     *
     * @param pageListReq 查询条件和分页信息
     * @return 查询结果
     */
    @Override
    public PageListInspectionCaseRes pageListQuery(PageListInspectionCaseReq pageListReq) {
        //TODO  还需支持 SKU信息、流出经销商、流入经销商 模糊搜索
        IPage<ListInspectionCaseVO> pageResult = inspectionCaseMapper.pageListQuery(PageUtil.createPage(pageListReq), pageListReq);
        List<ListInspectionCaseVO> inspectionCaseVOList = pageResult.getRecords();

        //转换类型
        List<PageListInspectionCaseRes.InspectionCaseBean> inspectionCaseBeanList = inspectionCaseVOList.stream()
                .parallel().map(s -> {
                    PageListInspectionCaseRes.InspectionCaseBean inspectionCaseBean = new PageListInspectionCaseRes.InspectionCaseBean();
                    BeanUtil.copyProperties(s, inspectionCaseBean);
                    //经销商
                    inspectionCaseBean.setOutAgency(s.getOutAgencyCode() + s.getOutAgencyName());
                    inspectionCaseBean.setInAgency(s.getInAgencyCode() + s.getInAgencyName());
                    //渠道
                    inspectionCaseBean.setChannel(ReqChannelTypeEnum.getName(s.getChannel()));
                    return inspectionCaseBean;
                }).collect(Collectors.toList());


        //封装出参
        PageListInspectionCaseRes pageListInspectionCaseRes = new PageListInspectionCaseRes();
        //放入分页信息
        PageUtil.copyPageInfo(pageListInspectionCaseRes, pageResult);
        pageListInspectionCaseRes.setInspectionCaseList(inspectionCaseBeanList);
        return pageListInspectionCaseRes;
    }

    /**
     * 编辑案件
     *
     * @param editReq 编辑内容
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public void editInspectionCase(EditInspectionCaseReq editReq) {

        final Long caseId = editReq.getId();
        //============校验箱码============
        //涉案箱码
        List<EditInspectionCaseReq.BoxCode> involvedBoxCodes = editReq.getInvolvedBoxCodes();
        List<String> boxCodeStrList = involvedBoxCodes.stream().map(EditInspectionCaseReq.BoxCode::getBoxCode).collect(Collectors.toList());
        //校验箱码信息  返回箱码对应流向信息
        Map map = this.validatedBoxCode(boxCodeStrList);


        //============修改案件信息============


        InspectionCase inspectionCase = new InspectionCase();
        //TODO 设置流入 流出经销商
        inspectionCase.setOutAgencyCode("Test_0001");
        inspectionCase.setOutAgencyName("Test_京城第一经销商");
        inspectionCase.setInAgencyCode("Test_0002");
        inspectionCase.setInAgencyName("Test_京城第二经销商");

        inspectionCase.setBoxCodeCount((long) involvedBoxCodes.size());
        inspectionCase.setId(caseId);
        inspectionCase.setCaseRemark(editReq.getCaseRemark());
        int i = inspectionCaseMapper.updateById(inspectionCase);
        //判断当前案件是否存在 i如果大于0表示 更新成功  否则表示未找到指定数据
        AssertUtil.isMeets(i, x -> x > 0, "当前案件已经不存在！");

        //============ 修改关联的箱码信息 ============
        //转换类型
        List<InspectionCaseBoxCode> inspectionCaseBoxCodeList = involvedBoxCodes.stream()
                .parallel().map(s -> {
                    InspectionCaseBoxCode inspectionCaseBoxCode = new InspectionCaseBoxCode();
                    BeanUtil.copyProperties(s, inspectionCaseBoxCode);
                    inspectionCaseBoxCode.setInspectionCaseId(caseId);
                    return inspectionCaseBoxCode;
                }).collect(Collectors.toList());

        //先删除当前案件下关联的箱码信息
        QueryWrapper<InspectionCaseBoxCode> boxCodeDeleteWrapper = new QueryWrapper<>();
        boxCodeDeleteWrapper.eq("inspection_case_id", editReq.getId());
        inspectionCaseBoxCodeMapper.delete(boxCodeDeleteWrapper);
        //批量保存
        inspectionCaseBoxCodeMapper.insertBatch(inspectionCaseBoxCodeList);

    }

    /**
     * 删除指定案件
     *
     * @param ids 指定案件ids
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public void removeInspectionCase(Long[] ids) {
        //先删除案件
        inspectionCaseMapper.deleteBatchIds(Arrays.asList(ids));
        //删除关联的箱码信息
        inspectionCaseBoxCodeMapper.deleteBatchCaseIds(ids);
    }

    /**
     * 获取案件详情信息
     *
     * @param id ID
     * @return 案件详情信息
     */
    @Override
    public GetInspectionCaseInfoRes getInspectionCaseInfo(final Long id) {
        InspectionCaseDetailVO caseDetailVO = inspectionCaseMapper.selectInspectionCaseDetailById(id);
        AssertUtil.isNull(caseDetailVO, "此案件未查询到！");

        GetInspectionCaseInfoRes getInspectionCaseInfoRes = new GetInspectionCaseInfoRes();
        BeanUtil.copyProperties(caseDetailVO, getInspectionCaseInfoRes);

        getInspectionCaseInfoRes.setOutAgency(caseDetailVO.getOutAgencyCode() + caseDetailVO.getOutAgencyName());
        getInspectionCaseInfoRes.setInAgency(caseDetailVO.getInAgencyCode() + caseDetailVO.getInAgencyName());

        return getInspectionCaseInfoRes;
    }


    /**
     * 校验箱码
     * 1、提交箱码的SKU信息是否一致，不一致则显示错误箱码；并列出错误原因
     * 2、提交箱码的流向信息是否一致
     *
     * @param boxCodeList 提交箱码列表
     * @return 流向信息
     */
    private Map validatedBoxCode(List<String> boxCodeList) {

        //TODO 查询流向信息
        //TODO  校验提交箱码所对应的SKU信息是否正确;、校验提交箱码的流向信息是否相同
        //List<String> errorBoxCodeList=new ArrayList<>();
        //校验失败 抛出异常 说明错误箱码；并列出错误原因
        //AssertUtil.isEmpty(errorBoxCodeList,"案件新增失败！箱码：XXXX；XXXXX，失败原因：SKU信息不一致/流向信息不一致");


        return MapUtil.empty();
    }

    /**
     * 自动生成案件编号
     *
     * @param provinceId 省份ID
     * @param cityId     城市ID
     * @return 生成的编号
     */
    private String autoGenerateCaseNumber(Long provinceId, Long cityId) {
        AssertUtil.isNull(provinceId, "省份ID不能为空");
        AssertUtil.isNull(cityId, "城市ID不能为空");
        return caseNumberService.getCaseNumber(cityId).toString();
    }


}
